# Programmes pédagogiques et versions

Le programme pédagogique, appelé *formation* dans ScoDoc, défini les unités
d'enseignement; il est destiné à être utilisé par plusieurs sessions de
formation (semestres). On doit apporter un soin particulier à la définition du
programme, et éviter de le modifier une fois que des semestres sont créés (il
est toutefois possible d'en créer de nouvelles *versions* pour permettre des
modifications ultérieures sans affecter les semestres achevés: voir plus bas.

Le programme pédagogique définit notamment les coefficients des modules qui le
composent. Les semestres qui se réfèrent à ce programme utilisent ces
coefficients pour calculer leurs notes. De même, les noms de UE et modules qui
apparaissent sur les bulletins viennent du programme. Il faut être
particulièrement vigilant lors des modifications du programme pédagogique.

Dans la configuration par défaut, seul le chef de département (rôle Admin) peut
modifier les programmes pédagogiques.

(voir aussi des exemples de programmes en bas de la page
[GuideAdminFormation](GuideAdminFormation.md)).

## Structure d'un programme pédagogique

On définira en général dans le programme l'ensemble des enseignements d'un
diplôme (les 4 semestres d'un DUT par exemple). C'est dans une phase ultérieure
que l'on mettra en place les différents semestres.

Les programmes pédagogiques ScoDoc sont structurés en Unités d'Enseignements
(UE), Matières et Modules. Un module appartient forcément à une matière, qui
appartient elle même à une UE. Les modules (déclinés en *ressources*, *SAÉs* en
BUT) représentent les cours ("mathématique", "anglais", ...) et sont associés à
un volume horaire (cours/TD/TP) et à un coefficient: chaque module produit une
note moyenne (en général obtenue à travers plusieurs *évaluations* ou
contrôles). La note moyenne d'une UE est obtenue en calculant une moyenne
pondérée par les coefficients des notes moyennes de modules.

🚸 En BUT, c'est un peu différent, puisque les modules ont une note *pour chaque
UE*: la moyenne d'un module n'est pas définie. Voir [BUT](BUT.md)

### Unités d'Enseignement (UE)

Les UE jouent un rôle particulier dans l'évaluation. En effet, selon les règles
du LMD, les UE sont *capitalisables* (voir
[CapitalisationUE](CapitalisationUE.md)). De plus, l'obtention de droit des
semestres d'un DUT est soumise à une moyenne supérieure à 8/20 dans chacune des
UE.

Notons qu'une UE ne possède pas de coefficient. Le coefficient d'une UE n'est
autre que la somme des coefficient des modules qui composent cette UE. Par
conséquent, le coefficient d'UE est potentiellement variable d'un étudiant à
l'autre, si les étudiants ne sont pas inscrits aux mêmes modules (options ou
parcours).

Note: une formation en plusieurs semestres devrait normalement avoir des UEs
différentes dans chaque semestre.

* Il est parfois désirable de capitaliser au sein d'un parcours des UE
  appartenant à deux programmes ScoDoc différents (par exemple, on peut avoir
  changé de version du programme entre deux semestres, comme expliqué plus
  loin). Dans ce cas, il faut attribuer aux programmes le même code de formation
  (via le lien "modifier" sur la page d'accueil des programmes), et aussi
  attribuer les mêmes codes aux UE (via le lien "modifier l'UE" sur la page
  "programme détaillé et semestres").

* Les UE peuvent être de type "normal" ou "Sport&Culture". Ces dernières ne sont
  utilisées que pour les notes optionnelles (activités culturelles et sportives)
  utilisée dans certains établissements. Elles se voient attribuer une règle de
  calcul spécifique qui dépend généralement de l'établissement (il n'y à pas de
  règle nationale pour la prise en compte des notes de sport et culture).
  Typiquement, la note des UE de ce type spécial agit directement sur la moyenne
  générale de l'étudiant.

### Matières

Les matières n'ont pas d'autre utilité que d'aider à structurer le programme.
Par exemple, on pourrait définir dans un programme une matière "Sciences"
réunissant les modules de "mathématiques" et de "physique". Les matières n'ont
pas de coefficient. Si l'on ne souhaite pas utiliser de matière, il suffit d'en
créer une pour chaque module avec le même nom, ou au contraire (plus simplement)
de créer une matière par UE et d'y placer tous les modules.

Note: les matières ne sont pas utilisées en BUT.

### Modules

* Le *code* du module va apparaitre sur les bulletins et certains tableaux
  récapitulatifs. Il comporte habituellement quelques caractères (comme "MATH",
  ou "SPO"). Si la version officielle de votre programme pédagogique n'utilise
  pas de codes de ce genre, inventez des codes à la fois courts (pas plus de 4
  ou 5 caractères) et évocateurs du nom du module.

* Le *titre* du module apparaitra sur le tableau de bord du semestre et sur les
  bulletins.

* L' *abréviation* est une version courte du titre. Si le titre n'est pas trop
  long (3 ou 4 mots), copier le. Sinon, inventer une abréviation en quelques
  mots qui soit lisible.

* Les volumes horaires ne sont présents que pour information et ne sont
  actuellement pas du tout utilisés par ScoDoc: il est donc facultatif de les
  indiquer.

* Le coefficient est utilisé pour le calcul de la moyenne d'UE et de la moyenne
  générale. Il s'agit d'un nombre réel positif ou nul.

* Choisir dans le menu la *matière* à laquelle appartient le module.

* Le semestre est un nombre indiquant dans quel semestre de la formation se
  place habituellement ce module. Il arrive que l'on décline la même formation
  selon différentes modalités (formation initiale, continue) avec des placements
  différents: dans ce cas, indiquer le semestre dans la modalité "habituelle";
  lors de la mise en place d'un semestre, on peut choisir manuellement des
  modules de tous les semestres.

### Ordre d'affichage des UE, matières et modules

Chaque élément (UE, matières et modules) possède un attribut *numéro* qui est un
nombre entier utilisé pour le classement des éléments de même niveau dans la
hiérarchie dans les tableaux et bulletins.

Il est conseillé d'attribuer les numéros de 10 en 10 afin de pouvoir plus
facilement insérer un nouvel élément entre deux éléments existants. Par exemple,
si l'on a dans une matière trois modules MA, MB, MC, on va leur attribuer les
numéros 10, 20 et 30.

## Verrouillage d'une formation

Lorsque au moins l'un des semestres qui se réfèrent à ce programme est
*verrouillé*, il devient impossible de modifier le programme (la page de
présentation du programme ne comporte alors aucun lien). Deux cas peuvent se
présenter:

* il s'agit d'une modification mineure (intitulé d'un module, ...) ne risquant
  pas affecter les notes existantes, et il y a peu de semestres verrouillés:
  dans ce cas, il est possible d'aller déverrouiller un à un les semestres
  concernés, puis d'effectuer la modification du programme avant de
  re-verrouiller les semestres.

* il s'agit d'une modification conséquente, on ne ne veut pas affecter les
  semestres existants: on crée alors une nouvelle *version* du programme. La
  version crée est une copie à l'identique du programme existant, que l'on peut
  modifier à sa guise.

## Versions de formation

Chaque semestre ("FormSemestre" dans le jargon ScoDoc) se réfère à un programme
pédagogique, appelé sa *formation*. cette formation définit l'ensemble des UE et
modules, leurs intitulés, et leurs coefficients et ECTS.

Les programmes sont le plus souvent adaptés localement, et peuvent varier d'une
année sur l'autre. Lorsqu'une formation est modifié (par exemple, un changement
de coefficient), ScoDoc doit recalculer l'ensemble des notes de tous les
semestres utilisant cette formation. De même, si un intitulé change, il faut
re-générer les bulletins et procès-verbaux. On conçoit donc que la modification
d'une formation ne s'aborde pas à la légère. ScoDoc empêche d'ailleurs toute
modification d'une formation (ou partie de, selon les cas) lorsqu'un semestre a
été verrouillé (ce qui indique en général qu'il est achevé et que l'on souhaite
conserver ses données et résultats inchangés pour utilisation future dans des
jurys ou avis).

Si vous devez modifier une formation pour la nouvelle année scolaire, vous
pouvez créer une nouvelle version d'une formation existante afin d'éviter
d'avoir à saisir de nouveau l'ensemble des éléments. Il arrive même que, l'année
scolaire déjà commencée, on se rende compte que l'on doit modifier la formation
d'un semestre en cours (bien sûr, cela ne devrait pas arriver, les modalités
d'évaluation étant souvent votées par des instances officielles avant le début
de l'année, mais le monde n'est pas parfait et de petites corrections sont
parfois nécessaires). Dans ce cas, ScoDoc vous permet d'associer un ou plusieurs
semestres existants dans une formation à une nouvelle version de celle-ci, créée
par copie.

<img src="/screens/menu-formsemestre-assoc.png" alt="menu formsemestre" width="33%">

La figure suivante montre la distinction entre formations et semestres, et les opérations possibles:

![menu formsemestre](fig/formations_versions_association.jpg)

![association nouvelle version](fig/sem-assoc-formation.png)

## Évolution du programme d'une année sur l'autre

Imaginons que nous ayons les semestres S1, S2, S3, S4 dans un programme V1, et
que l'année suivante une nouvelle version du programme entre en vigueur
(adaptation locale, corrections diverses...).

Voici comment mettre en place les semestres de l'année suivante tout en
conservant le maximum d'éléments (évaluation, choix des ressources et SAE).

1. Cloner (menu **Semestre / Cloner**)les semestres concernés (S1 à S4 ici)
2. Prendre l'un des semestres copiés (peu importe laquelle), et suivre
**Semestre / Associer à une nouvelle version**. Il est alors possible d'associer
à cette nouvelle version (V2) les semestres: cochez les semestres créés à
l'étape précédente.

Cette façon de procéder permet de limiter le nombre de nouvelles versions de
formations créée.

<img src="/screens/formsemestre_associate_new_version.png" width="100%">

## Changement de la formation d'un semestre

Il peut arriver que l'on ai créé deux versions de formations, qui sont encore
identique, et que l'on souhaite rattacher un formsemestre de l'une à l'autre.
C'est possible, à condition que les deux formations soient vraiment identiques
(mêmes UEs, titres, coefficients, etc). Le lien est accessible en bas de la page
"Associer à une nouvelle version du programme" mentionnée ci-dessus.

![change formation](fig/sem-change-formation.png)

!!! note "Voir aussi"

    - [Guide du responsable de formation](GuideAdminFormation.md)
    - [Guide utilisateur](GuideUtilisateur.md)
    - Pour le BUT: [le référentiel de compétence](BUTReferentielCompetences.md)
    - [Tutoriels vidéo](https://www.youtube.com/channel/UCb0JYCBRi0CsE4XFp4ByhXg)
          - [Gestion des UE Bonus](https://www.youtube.com/watch?v=SVbjuDpq-lI)
          - [Mise en place des parcours BUT](https://www.youtube.com/watch?v=OnuOXJo-3ro)
          - [Saisie des codes Apogée](https://www.youtube.com/watch?v=MW0nNhbBjDM)
          - [Du DUT au BUT: comment transformer un programme](https://www.youtube.com/watch?v=9HizGTvYgck)
    - [FAQ](FAQ.md)
    - [Contacts](Contact.md)
